/*	
**	
**	Copyright (C) 1999 Geert Bevin
**	gbevin@thunderstorms.org
**	http://thunderstorms.org/fm
**
**	This program is free software; you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation; either version 2 of the License, or
**	(at your option) any later version.
**	
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**	
**	You should have received a copy of the GNU General Public License
**	along with this program; if not, write to the Free Software
**	
**	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**		
*/

#include <unistd.h>
#include <stdio.h>
#include <glib.h>
#include <fcntl.h>
#include <sys/stat.h>

#include "ctreecolumn.h"
#include "fmwindow.h"
#include "layout.h"
#include "preferences.h"

extern GList	*pgFmWindows;
extern gchar	*pgConfigPath;

static void 				layout_entryClear(struct LayoutEntry*);
static void 				layout_entryDestroy(struct LayoutEntry*);
static struct LayoutEntry*	layout_entryNew();
static void					layout_entryProcess(struct LayoutEntry*);
static gchar*				layout_getFileName();
static guint				layout_parseSymbol(GScanner*);

enum
{
	SYMBOL_BROWSER			= G_TOKEN_LAST + 1,
	
	SYMBOL_GEOMETRY			= G_TOKEN_LAST + 2,
	SYMBOL_APPEARANCE		= G_TOKEN_LAST + 3,
	SYMBOL_BOOKMARK			= G_TOKEN_LAST + 4,
	SYMBOL_COLUMN			= G_TOKEN_LAST + 5,
	SYMBOL_EXPANDEDDIR		= G_TOKEN_LAST + 6,
	SYMBOL_COMBOENTRY		= G_TOKEN_LAST + 7,
	
	SYMBOL_X				= G_TOKEN_LAST + 8,
	SYMBOL_Y				= G_TOKEN_LAST + 9,
	SYMBOL_WIDTH			= G_TOKEN_LAST + 10,
	SYMBOL_HEIGHT			= G_TOKEN_LAST + 11,
	
	SYMBOL_FOCUS			= G_TOKEN_LAST + 12,
	SYMBOL_HIDEDOTFILES		= G_TOKEN_LAST + 13,
	SYMBOL_SORTTYPE			= G_TOKEN_LAST + 14,
	SYMBOL_SORTDIRECTION	= G_TOKEN_LAST + 15,
	SYMBOL_ZOOM				= G_TOKEN_LAST + 16,
	
	SYMBOL_ID				= G_TOKEN_LAST + 17,
	SYMBOL_PATH				= G_TOKEN_LAST + 18
};

static const struct SymbolDefinition	psSymbolTableLayout[] =
{
	{ "BROWSER", 		SYMBOL_BROWSER },

	{ "geometry", 		SYMBOL_GEOMETRY },
	{ "appearance",    	SYMBOL_APPEARANCE },
	{ "bookmark", 		SYMBOL_BOOKMARK },
	{ "column", 		SYMBOL_COLUMN },
	{ "expandeddir",	SYMBOL_EXPANDEDDIR },
	{ "comboentry",		SYMBOL_COMBOENTRY },
	
	{ "Xpos", 	  		SYMBOL_X },
	{ "Ypos", 	  		SYMBOL_Y },
	{ "Width", 	  		SYMBOL_WIDTH },
	{ "Height", 	  	SYMBOL_HEIGHT },
	
	{ "Focus", 	  		SYMBOL_FOCUS },
	{ "HideDotFiles", 	SYMBOL_HIDEDOTFILES },
	{ "SortType", 	  	SYMBOL_SORTTYPE },
	{ "SortDirection", 	SYMBOL_SORTDIRECTION },
	{ "Zoom", 			SYMBOL_ZOOM },
	
	{ "Id",				SYMBOL_ID },
	{ "Path",			SYMBOL_PATH },
	
	{ NULL, 			0 } 
};

void layout_clear()
{
	gchar	*layout_filename = layout_getFileName();

	if(layout_filename)
	{
		unlink(layout_filename);
		g_free(layout_filename);
	}
}

static void layout_entryClear(struct LayoutEntry *layoutEntry)
{
	if(layoutEntry->name == SYMBOL_GEOMETRY)
	{
        layoutEntry->values.geometry.x = 0;
        layoutEntry->values.geometry.y = 0;
        layoutEntry->values.geometry.width = 0;
        layoutEntry->values.geometry.height = 0;
	}
	else if(layoutEntry->name == SYMBOL_APPEARANCE)
	{
		if(layoutEntry->values.appearance.focus)
		{
			g_free(layoutEntry->values.appearance.focus);
            layoutEntry->values.appearance.focus = NULL;
		}
		if(layoutEntry->values.appearance.zoom)
		{
			g_free(layoutEntry->values.appearance.zoom);
            layoutEntry->values.appearance.zoom = NULL;
		}
        layoutEntry->values.appearance.hideDotFiles = FALSE;
        layoutEntry->values.appearance.sortType = FALSE;
        layoutEntry->values.appearance.sortDirection = FALSE;
	}
	else if(layoutEntry->name == SYMBOL_BOOKMARK)
	{
        layoutEntry->values.bookmark.id = 0;
		if(layoutEntry->values.bookmark.path)
		{
			layoutEntry->values.bookmark.path = NULL;
		}
	}
	else if(layoutEntry->name == SYMBOL_COLUMN)
	{
        layoutEntry->values.column.id = 0;
	}
	else if(layoutEntry->name == SYMBOL_EXPANDEDDIR)
	{
		if(layoutEntry->values.expandeddir.path)
		{
			g_free(layoutEntry->values.expandeddir.path);
            layoutEntry->values.expandeddir.path = NULL;
		}
	}
	else if(layoutEntry->name == SYMBOL_COMBOENTRY)
	{
		if(layoutEntry->values.comboEntry.path)
		{
			g_free(layoutEntry->values.comboEntry.path);
            layoutEntry->values.comboEntry.path = NULL;
		}
	}
}

static void layout_entryDestroy(struct LayoutEntry *layoutEntry)
{
    layout_entryClear(layoutEntry);
	g_free(layoutEntry);
}

static struct LayoutEntry* layout_entryNew()
{
	struct LayoutEntry *layout_entry = NULL;

    layout_entry = g_new0(struct LayoutEntry, 1);
    layout_entryClear(layout_entry);

	return layout_entry;
}

static void layout_entryProcess(struct LayoutEntry *layoutEntry)
{
	// check if there is a valid previous preference entry
	if(layoutEntry && layoutEntry->fmWindow)
	{
		if(layoutEntry->name == SYMBOL_GEOMETRY)
		{
            layoutEntry->fmWindow->initialX	= layoutEntry->values.geometry.x;
			layoutEntry->fmWindow->initialY	= layoutEntry->values.geometry.y;
			layoutEntry->fmWindow->initialWidth	= layoutEntry->values.geometry.width;
			layoutEntry->fmWindow->initialHeight	= layoutEntry->values.geometry.height;
//			printf("Defined GEOMETRY : %d, %d, %d, %d\n", layoutEntry->values.geometry.x, layoutEntry->values.geometry.y, layoutEntry->values.geometry.width, layoutEntry->values.geometry.height);
		}
		else if(layoutEntry->name == SYMBOL_APPEARANCE)
		{
			if(layoutEntry->values.appearance.focus)
			{
				layoutEntry->fmWindow->pInitialFocus = g_strdup(layoutEntry->values.appearance.focus);
			}
			if(layoutEntry->values.appearance.zoom)
			{
				if(layoutEntry->fmWindow->pZoomDirectory)
				{
					g_free(layoutEntry->fmWindow->pZoomDirectory);
				}
				layoutEntry->fmWindow->pZoomDirectory = g_strdup(layoutEntry->values.appearance.zoom);
			}
            layoutEntry->fmWindow->hideDotFiles = layoutEntry->values.appearance.hideDotFiles;
			layoutEntry->fmWindow->sortDirection = layoutEntry->values.appearance.sortDirection;
			layoutEntry->fmWindow->sortType = layoutEntry->values.appearance.sortType;
//			printf("Defined APPEARANCE : '%s', %d, %d, %d\n", layoutEntry->values.appearance.focus, layoutEntry->values.appearance.hideDotFiles, layoutEntry->values.appearance.sortDirection, layoutEntry->values.appearance.sortType);
		}
		else if(layoutEntry->name == SYMBOL_BOOKMARK && layoutEntry->values.bookmark.path)
		{
            layoutEntry->fmWindow->pBookmarks[layoutEntry->values.bookmark.id] = g_strdup(layoutEntry->values.bookmark.path);
//			printf("Defined BOOKMARK : %d, '%s'\n", layoutEntry->values.bookmark.id, layoutEntry->values.bookmark.path);
		}
		else if(layoutEntry->name == SYMBOL_COLUMN)
		{
            layoutEntry->fmWindow->pCTreeColumns = g_list_prepend(layoutEntry->fmWindow->pCTreeColumns, (gpointer)layoutEntry->values.column.id);
//			printf("Defined COLUMN : %d\n", layoutEntry->values.column.id);
		}
		else if(layoutEntry->name == SYMBOL_EXPANDEDDIR && layoutEntry->values.expandeddir.path)
		{
            layoutEntry->fmWindow->pExpandedDirs = g_list_prepend(layoutEntry->fmWindow->pExpandedDirs, g_strdup(layoutEntry->values.expandeddir.path));
//			printf("Defined EXPANDEDDIR : '%s'\n", layoutEntry->values.expandeddir.path);
		}
		else if(layoutEntry->name == SYMBOL_COMBOENTRY && layoutEntry->values.comboEntry.path)
		{
            layoutEntry->fmWindow->pComboBoxStrings = g_list_append(layoutEntry->fmWindow->pComboBoxStrings, g_strdup(layoutEntry->values.comboEntry.path));
//			printf("Defined COMBOENTRY : '%s'\n", layoutEntry->values.comboEntry.path);
		}
	}
}

static gchar* layout_getFileName()
{
	gchar	*layout_filename = g_strjoin("/", pgConfigPath, "layout", NULL);

	return layout_filename;
}

void layout_load()
{
	gchar							*layout_filename = NULL;
	int								layout_file_descriptor = -1;
	GScanner						*scanner = NULL;
	gchar							*cset_identifier_nth = NULL;
	const struct SymbolDefinition	*symbol_table_pointer = psSymbolTableLayout;
	guint							expected_token = G_TOKEN_NONE;


	layout_filename = layout_getFileName();
	if(layout_filename)
	{
		layout_file_descriptor = open(layout_filename, O_RDONLY);
		if(layout_file_descriptor != -1)
		{
			scanner = g_scanner_new(NULL);
			if(scanner)
			{
                scanner->config->case_sensitive = FALSE;
                scanner->config->numbers_2_int = TRUE;
                scanner->config->symbol_2_token = TRUE;
				if(scanner->config->cset_identifier_nth)
				{
                    cset_identifier_nth = g_strjoin("", scanner->config->cset_identifier_nth, ".", NULL);
                    scanner->config->cset_identifier_nth = cset_identifier_nth;
				}

				while(symbol_table_pointer->name)
				{
//					printf("SYMBOL %s=%d\n", symbol_table_pointer->name, symbol_table_pointer->token);
					g_scanner_add_symbol(scanner, symbol_table_pointer->name, GINT_TO_POINTER(symbol_table_pointer->token));
                    symbol_table_pointer++;
				}

				g_scanner_input_file(scanner, layout_file_descriptor);

				// parse the file
				do
				{
					expected_token = layout_parseSymbol(scanner);
					g_scanner_peek_next_token(scanner);
				}
				while(expected_token != G_TOKEN_NONE && expected_token != G_TOKEN_EOF && expected_token != G_TOKEN_ERROR);

				g_scanner_destroy(scanner);
			}
			else
			{
				printf("layout_load : Can't create a new GScanner.\n");
			}
			close(layout_file_descriptor);
			g_free(layout_filename);
		}
 		else
		{
//			printf("layout_load : Can't open the layout file '%s' for reading.\n", layout_filename);
			g_free(layout_filename);
		}
	}
	else
	{
		printf("layout_load : Can't create the layout filename\n");
	}
}

static guint layout_parseSymbol(GScanner *scanner)
{
	static struct LayoutEntry	*layout_entry = NULL;
	guint  			 			previous_token = G_TOKEN_NONE;

	g_scanner_get_next_token(scanner);
//	printf("scanner->token %d\n", scanner->token);
	
	// check for end of file and for error
	if(scanner->token == G_TOKEN_EOF || scanner->token == G_TOKEN_ERROR)
	{
		layout_entryProcess(layout_entry);
		if(layout_entry)
		{
			if(layout_entry->fmWindow)
			{
				buildNewFmWindow(layout_entry->fmWindow);
			}
			layout_entryDestroy(layout_entry);
		}
		return scanner->token;
	}
	else if(scanner->token == G_TOKEN_COMMENT_SINGLE || scanner->token == G_TOKEN_COMMENT_MULTI)
	{
		return scanner->token;
	}
	// get browser
	else if(scanner->token == G_TOKEN_LEFT_BRACE)
	{
		g_scanner_get_next_token(scanner);
//		printf("scanner->token %d\n", scanner->token);
		if(scanner->token == SYMBOL_BROWSER)
		{
			g_scanner_peek_next_token(scanner);
			if(scanner->next_token == G_TOKEN_RIGHT_BRACE)
			{
                layout_entryProcess(layout_entry);
				if(layout_entry)
				{
					if(layout_entry->fmWindow)
					{
						buildNewFmWindow(layout_entry->fmWindow);
					}
					layout_entryDestroy(layout_entry);
				}
				layout_entry = layout_entryNew();
				if(!layout_entry)
				{
					printf("layout_parseSymbol : FATAL ERROR, can't allocate a new layout entry, quitting.\n");
					exit(1);
				}
				layout_entry->fmWindow = fmWindow_new(NULL);

				g_scanner_get_next_token(scanner);
//				printf("scanner->token %d\n", scanner->token);

//				printf("Started new BROWSER\n");
				
				return SYMBOL_BROWSER;
			}
		}
		// get name
		else if(scanner->token == SYMBOL_GEOMETRY || 
				scanner->token == SYMBOL_APPEARANCE ||
				scanner->token == SYMBOL_BOOKMARK ||
				scanner->token == SYMBOL_COLUMN ||
				scanner->token == SYMBOL_EXPANDEDDIR ||
				scanner->token == SYMBOL_COMBOENTRY)
		{
			g_scanner_peek_next_token(scanner);
			if(scanner->next_token == G_TOKEN_RIGHT_BRACE)
			{
                layout_entryProcess(layout_entry);
				// initialize a new preference entry
				layout_entryClear(layout_entry);
				layout_entry->name = scanner->token;

				g_scanner_get_next_token(scanner);
//				printf("scanner->token %d\n", scanner->token);

//				printf("Started new NAME %d\n", layout_entry->name);

				return layout_entry->name;
			}
		}
	}
	// get the name values
	else
	{
		if(layout_entry->name == SYMBOL_GEOMETRY)
		{
			if(scanner->token == SYMBOL_X ||
               scanner->token == SYMBOL_Y ||
               scanner->token == SYMBOL_WIDTH ||
			   scanner->token == SYMBOL_HEIGHT)
			{
				g_scanner_peek_next_token(scanner);
				if(scanner->next_token == G_TOKEN_EQUAL_SIGN)
				{
					previous_token = scanner->token;
					g_scanner_get_next_token(scanner);
//					printf("scanner->token %d\n", scanner->token);
					g_scanner_get_next_token(scanner);
//					printf("scanner->token %d\n", scanner->token);
					switch(previous_token)
					{
						case SYMBOL_X :
							if(scanner->token == G_TOKEN_INT)
							{
								layout_entry->values.geometry.x = scanner->value.v_int;
								return previous_token;
							}
							break;
						case SYMBOL_Y :
							if(scanner->token == G_TOKEN_INT)
							{
								layout_entry->values.geometry.y = scanner->value.v_int;
								return previous_token;
							}
							break;
						case SYMBOL_WIDTH :
							if(scanner->token == G_TOKEN_INT)
							{
								layout_entry->values.geometry.width = scanner->value.v_int;
								return previous_token;
							}
							break;
						case SYMBOL_HEIGHT :
							if(scanner->token == G_TOKEN_INT)
							{
								layout_entry->values.geometry.height = scanner->value.v_int;
								return previous_token;
							}
							break;
					}
				}
			}
		}
		else if(layout_entry->name == SYMBOL_APPEARANCE)
		{
			if(scanner->token == SYMBOL_FOCUS ||
               scanner->token == SYMBOL_HIDEDOTFILES ||
               scanner->token == SYMBOL_SORTTYPE ||
               scanner->token == SYMBOL_SORTDIRECTION ||
			   scanner->token == SYMBOL_ZOOM)
			{
				g_scanner_peek_next_token(scanner);
				if(scanner->next_token == G_TOKEN_EQUAL_SIGN)
				{
					previous_token = scanner->token;
					g_scanner_get_next_token(scanner);
//					printf("scanner->token %d\n", scanner->token);
					g_scanner_get_next_token(scanner);
//					printf("scanner->token %d\n", scanner->token);
					switch(previous_token)
					{
						case SYMBOL_FOCUS :
							if(scanner->token == G_TOKEN_STRING)
							{
								layout_entry->values.appearance.focus = g_strdup(scanner->value.v_string);
								return previous_token;
							}
							break;
						case SYMBOL_HIDEDOTFILES :
							if(scanner->token == G_TOKEN_INT)
							{
								layout_entry->values.appearance.hideDotFiles = scanner->value.v_int;
								return previous_token;
							}
							break;
						case SYMBOL_SORTTYPE :
							if(scanner->token == G_TOKEN_INT)
							{
								layout_entry->values.appearance.sortType = scanner->value.v_int;
								return previous_token;
							}
							break;
						case SYMBOL_SORTDIRECTION :
							if(scanner->token == G_TOKEN_INT)
							{
								layout_entry->values.appearance.sortDirection = scanner->value.v_int;
								return previous_token;
							}
							break;
						case SYMBOL_ZOOM :
							if(scanner->token == G_TOKEN_STRING)
							{
								layout_entry->values.appearance.zoom = g_strdup(scanner->value.v_string);
								return previous_token;
							}
							break;
					}
				}
			}
		}
		else if(layout_entry->name == SYMBOL_BOOKMARK)
		{
			if(scanner->token == SYMBOL_ID ||
               scanner->token == SYMBOL_PATH)
			{
				g_scanner_peek_next_token(scanner);
				if(scanner->next_token == G_TOKEN_EQUAL_SIGN)
				{
					previous_token = scanner->token;
					g_scanner_get_next_token(scanner);
//					printf("scanner->token %d\n", scanner->token);
					g_scanner_get_next_token(scanner);
//					printf("scanner->token %d\n", scanner->token);
					switch(previous_token)
					{
						case SYMBOL_ID :
							if(scanner->token == G_TOKEN_INT)
							{
								layout_entry->values.bookmark.id = scanner->value.v_int;
								return previous_token;
							}
							break;
						case SYMBOL_PATH :
							if(scanner->token == G_TOKEN_STRING)
							{
								layout_entry->values.bookmark.path = g_strdup(scanner->value.v_string);
								return previous_token;
							}
							break;
					}
				}
			}
		}
		else if(layout_entry->name == SYMBOL_COLUMN)
		{
			if(scanner->token == SYMBOL_ID)
			{
				g_scanner_peek_next_token(scanner);
				if(scanner->next_token == G_TOKEN_EQUAL_SIGN)
				{
					previous_token = scanner->token;
					g_scanner_get_next_token(scanner);
//					printf("scanner->token %d\n", scanner->token);
					g_scanner_get_next_token(scanner);
//					printf("scanner->token %d\n", scanner->token);
					switch(previous_token)
					{
						case SYMBOL_ID :
							if(scanner->token == G_TOKEN_INT)
							{
								layout_entry->values.column.id = scanner->value.v_int;
								return previous_token;
							}
							break;
					}
				}
			}
		}
		else if(layout_entry->name == SYMBOL_EXPANDEDDIR)
		{
			if(scanner->token == SYMBOL_PATH)
			{
				g_scanner_peek_next_token(scanner);
				if(scanner->next_token == G_TOKEN_EQUAL_SIGN)
				{
					previous_token = scanner->token;
					g_scanner_get_next_token(scanner);
//					printf("scanner->token %d\n", scanner->token);
					g_scanner_get_next_token(scanner);
//					printf("scanner->token %d\n", scanner->token);
					switch(previous_token)
					{
						case SYMBOL_PATH :
							if(scanner->token == G_TOKEN_STRING)
							{
								layout_entry->values.expandeddir.path = g_strdup(scanner->value.v_string);
								return previous_token;
							}
							break;
					}
				}
			}
		}
		else if(layout_entry->name == SYMBOL_COMBOENTRY)
		{
			if(scanner->token == SYMBOL_PATH)
			{
				g_scanner_peek_next_token(scanner);
				if(scanner->next_token == G_TOKEN_EQUAL_SIGN)
				{
					previous_token = scanner->token;
					g_scanner_get_next_token(scanner);
//					printf("scanner->token %d\n", scanner->token);
					g_scanner_get_next_token(scanner);
//					printf("scanner->token %d\n", scanner->token);
					switch(previous_token)
					{
						case SYMBOL_PATH :
							if(scanner->token == G_TOKEN_STRING)
							{
								layout_entry->values.comboEntry.path = g_strdup(scanner->value.v_string);
								return previous_token;
							}
							break;
					}
				}
			}
		}
	}

	return G_TOKEN_NONE;
}

void layout_save(struct Preferences *preferences)
{
	gchar			*layout_filename = NULL;
	FILE			*layout_file_stream = NULL;
	guint			counter = 0;
	struct FmWindow	*fm_window = NULL;
	guint			x = 0;
	guint			y = 0;
	guint			width = 0;
	guint			height = 0;
	guint			counter2 = 0;
	guint			column_id = 0;

	layout_filename = layout_getFileName();
	if(layout_filename)
	{
		unlink(layout_filename);
		layout_file_stream = fopen(layout_filename, "w");
		if(layout_file_stream)
		{
			for(counter = 0; counter < g_list_length(pgFmWindows); counter++)
			{
				fm_window = (struct FmWindow *)g_list_nth_data(pgFmWindows, counter);
				fprintf(layout_file_stream, "[BROWSER]\n");
				if(preferences->sessionSaveLayoutGeometry)
				{
					fprintf(layout_file_stream, "\t[geometry]\n");
					gdk_window_get_position(fm_window->pWindow->window, &x, &y);
					gdk_window_get_size(fm_window->pWindow->window, &width, &height);
					fprintf(layout_file_stream, "\t\tXpos=%d\n", x);
					fprintf(layout_file_stream, "\t\tYpos=%d\n", y);
					fprintf(layout_file_stream, "\t\tWidth=%d\n", width);
					fprintf(layout_file_stream, "\t\tHeight=%d\n", height);
				}
                if(preferences->sessionSaveLayoutBrowserAppearance)
				{
					fprintf(layout_file_stream, "\t[appearance]\n");
					fprintf(layout_file_stream, "\t\tFocus='%s'\n", fmWindow_getFilenameToRemember(fm_window, fm_window->pFocusedNode));
					fprintf(layout_file_stream, "\t\tHideDotFiles=%d\n", fm_window->hideDotFiles);
					fprintf(layout_file_stream, "\t\tSortType=%d\n", fm_window->sortType);
					fprintf(layout_file_stream, "\t\tSortDirection=%d\n", fm_window->sortDirection);
					fprintf(layout_file_stream, "\t\tZoom='%s'\n", fm_window->pZoomDirectory);
				}
                if(preferences->sessionSaveLayoutBookmarks)
				{
					for(counter2 = 0; counter2 < 10; counter2++)
					{
						if(fm_window->pBookmarks[counter2])
						{
							fprintf(layout_file_stream, "\t[bookmark]\n");
							fprintf(layout_file_stream, "\t\tId=%d\n", counter2);
							fprintf(layout_file_stream, "\t\tPath='%s'\n", fm_window->pBookmarks[counter2]);
						}
					}
				}
                if(preferences->sessionSaveLayoutBrowserAppearance)
				{
					for(counter2 = 0; counter2 < g_list_length(fm_window->pCTreeColumns); counter2++)
					{
						column_id = (guint)g_list_nth_data(fm_window->pCTreeColumns, counter2);
						if(column_id && column_id != COLUMN_BOOKMARK && column_id != COLUMN_FILENAME)
						{
							fprintf(layout_file_stream, "\t[column]\n");
							fprintf(layout_file_stream, "\t\tId=%d\n", column_id);
						}
					}
				}
                if(preferences->sessionSaveLayoutExpandedDirs)
				{
					for(counter2 = 0; counter2 < g_list_length(fm_window->pExpandedDirs); counter2++)
					{
						if(g_list_nth_data(fm_window->pExpandedDirs, counter2))
						{
							fprintf(layout_file_stream, "\t[expandeddir]\n");
							fprintf(layout_file_stream, "\t\tPath='%s'\n", (gchar *)g_list_nth_data(fm_window->pExpandedDirs, counter2));
						}
					}
				}
                if(preferences->sessionSaveLayoutComboHistory)
				{
					for(counter2 = 0; counter2 < g_list_length(fm_window->pExpandedDirs); counter2++)
					{
						if(g_list_nth_data(fm_window->pComboBoxStrings, counter2))
						{
							fprintf(layout_file_stream, "\t[comboentry]\n");
							fprintf(layout_file_stream, "\t\tPath='%s'\n", (gchar *)g_list_nth_data(fm_window->pComboBoxStrings, counter2));
						}
					}
				}
			}
			fclose(layout_file_stream);
			g_free(layout_filename);
		}
		else
		{
			printf("layout_save : Can't open the layout file '%s' for writing.\n", layout_filename);
		}
	}
	else
	{
		printf("layout_save : Can't create the layout filename\n");
	}
}

